#!/bin/bash
# SafeSquid Setup Script Build - 11:56 AM Thursday, September 06, 2007

CWD=`dirname $0`
IAM=`basename $0`
CURTIME=`date +%Y%m%d%H%M%S`
BACKUP_DIR="${CWD}/BACKUP-${CURTIME}"
COLUMNS=`echo ${COLUMNS:-80}`
INSTALL_LOG=${CWD}/${CURTIME}.safesquid.install.log

WAIT ()
	{
	
	while [ 1 ]
	do
		echo "Press \"Y\" to continue"
		echo "Press \"E\" to exit & terminate installation now"
		
		unset P; read -e -n 1 R; P=`echo ${R} | tr a-z A-Z`
		case $P in
			Y) break;
			;;
			E) exit;
			;;
			*) continue;
		esac
	done

	
	
	}

TOP_LINE ()
	{
		# Create the Borders with any character specified by "Border_Style"
		unset BORDER;
		typeset -i n; n=$[ ${COLUMNS} + 0 ]
		while [ $n -ge 1 ] 
			do BORDER="${BORDER}""${BORDER_STYLE}" 
				n=$[ $n - 1 ]
			done
	return;
	}

OPTION_CODE ()
	{
		# Print with Bright Yellow characters on Black background.
		# Generally used to display the Option Keys, to quickly reach a particular section of the Installation
		[ "x${*}" != "x" ] && echo -ne "\E[33;40m""${*}"
#		tput sgr0
	return;
	}
	
SHOW_BANNER ()
	{
		# Print the Top Banner that gets displayed throughout the Installation Process
		clear
		BORDER_STYLE="_" && TOP_LINE ;echo -e '\E[30;44m'"${BORDER}"; tput sgr0
		BORDER_STYLE=" " && TOP_LINE ;echo -e '\E[37;44m'"${BORDER}"; tput sgr0
		echo -ne '\E[37;44m'"   Installing SafeSquid: Content Filtering Internet Proxy   "; tput sgr0 ;
#		echo -ne "  Installing - SafeSquid: Content Filtering Internet Proxy  "; tput sgr0 ;
		echo -ne "  Version: ";OPTION_CODE "${VERSION}"; tput sgr0 ;
		echo -ne "  Edition: ";OPTION_CODE "${EDITION}";
		echo -ne "\n"; tput sgr0
		BORDER_STYLE="_" && TOP_LINE ;echo -e '\E[30;44m'"${BORDER}"; tput sgr0
		BORDER_STYLE=" " && TOP_LINE ;echo -e '\E[37;44m'"${BORDER}"; tput sgr0
	return;
	}
	
READ_CONF ()
	{
	# This function should always be called as READ_CONF "FILE_NAME"
	# "FILE_NAME" will be parsed to set the PARAMS
		PARAMS=`grep -ivE "#" $1`; eval "${PARAMS}"
		VARS=( `grep -vE "#" $1 | grep -iE "=" | awk 'BEGIN {FS="="} {print $1}'` )
		echo "${#VARS[@]} to be adjusted"
		let "M = ${#VARS[@]} - 1"
#		PROG=${INSTANCE_DIR}/${INSTANCE}
#		VER=( `eval "${PROG}" 2>&1 | grep "VERSION"` ) && EDITION=`echo ${VER[0]} | tr 'a-z' 'A-Z'`
	return;
	}
	
QUERY_OLD_CONF ()
	{
		SHOW_BANNER
		INPUT=""
#		OLD_CONF=""
		echo ""
		if [ "x${OLD_CONF}" != "x" ] && [ -f ${OLD_CONF} ] ; then
			echo -ne "Found "
			OPTION_CODE ${OLD_CONF}; tput sgr0
			cat <<- _EOF
			
			Press "C" to if this is Correct and you want to edit this file 
			Press Any Other Key to select Another File"
			_EOF
			read -e -n 1 INPUT ; P=`echo ${INPUT} | tr a-z A-Z`
			[ "x${P}" == "xC" ] && return;
		fi
		echo -ne "Please type the full path for your existing startup.conf and press Enter""\n"
		read INPUT
		if [ "x${INPUT}" != "x" ] ; then
			echo -ne "searching for ${INPUT} "
			if [ -f ${INPUT} ] ; then
				OLD_CONF=${INPUT} && echo "... Success " 
			else 
				echo " ... Failed, try again ! "
			OLD_CONF=""
			fi
		fi
		WAIT
		[ "x${OLD_CONF}" == "x" ] && QUERY_OLD_CONF
		
	return;
	}

QUERY_PURPOSE ()
	{
	while [ 1 ]
	do
	
	SHOW_BANNER
	# Identify the purpose of this Installation
		cat <<- _EOF
		
			Press "F" if we are doing a Fresh installation
			Press "U" if we want to Update an existing installation
			Press "A" if we want to Adjust an existing conf file
			
		_EOF

		read -n 1 INPUT ; P=`echo ${INPUT} | tr a-z A-Z`
		case ${P} in
		F)
			PURPOSE="FRESH_INSTALL"
			break;
		;;
		U)
			PURPOSE="UPGRADE"
			break;
		;;
		A)
			PURPOSE="ADJUST"
			break;
		;;
		*)
			continue;
		;;
		esac
		
	break
	done
	
	return;
	}

SSL_WORK_AROUND ()
	{
		echo -ne "Trying to locate libssl in /usr/lib/ ... "
		ls /usr/lib/*ssl* 2>/dev/null
		if [ $? -gt 0 ] ; then
			echo "Nothing Found!"
			echo "Please install openssl-0.9.7 or higher"
		else
			LIB=( `ls /usr/lib/*ssl* 2>/dev/null` )
			for TRY_LIB in `echo -ne ${LIB[*]}`
			do
				echo "Soft-linking ${MISSING} to ${TRY_LIB} "
				LIBDIR=`dirname ${TRY_LIB}`
				ln -s ${TRY_LIB} ${LIBDIR}/${MISSING}
				echo -ne "Checking if ${TRY_LIB} can work for us ... "
				ldd ${NEW_EXE} | grep -i "not found" | grep -i "${MISSING}" || echo "Yes it seems to work" ; break
				echo "No, it does not"
				echo -ne "Reversing the change ... "
				rm -f ${LIBDIR}/${MISSING} && echo "Done"
			done
		fi
	return;
	}
		

FIND_WORK_AROUND ()
	{
		case $MISSING in
		libpam.so.0 ) echo "Please install pam-0.77 or higher"
		;;
		libdl.so.2|libpthread.so.0|libm.so.6|libc.so.6 ) echo "Please install glibc-2.3.6 or higher"
		;;
		libstdc++.so.5 ) echo "Please install libstdc++-3.4.4 or higher"
		;;
		libgcc_s.so.1 ) echo "Please install libgcc-3.4.4 or higher"
		;;
		libssl.so.4 )  SSL_WORK_AROUND
		;;
		* ) echo "This is an unknown problem!"
			echo "Please inform SafeSquid Tech Support at http://www.safesquid.com/"
		;;
		esac
	return;
	}


	
CHECK_LIB ()
	{
		LIBRARY=( `objdump -p ${NEW_EXE} | awk '/NEEDED/ {print $2}'` )
		let n=0
		for NEEDS in `echo -ne ${LIBRARY[*]}`
		do
			echo -ne "Locating Library: ${LIBRARY[$n]} ... "
			DISKFILE[$n]=`whereis ${LIBRARY[$n]} | awk '{print $2}'`
			if [ "x${DISKFILE[$n]}" != "x" ] ; then
				echo "Success!"
			else
				MISSING=`echo ${LIBRARY[$n]}`
				echo "Not Found!"
				FIND_WORK_AROUND
			fi
		let "n = n + 1"
		done
	return;
	}
	
CHECK_NEW ()
	{
		SHOW_BANNER
	# Locate the new executable binary
		NEW_EXE=`dir -1t ${CWD}/bin/safesquid* | head -n 1` && echo "Found ${NEW_EXE}"
		[ "x${NEW_EXE}" == "x" ] && echo "Executable Binary to install not found in ${CWD}/bin/" && exit
	# Identify the version of the new binary
		NEW_EXE_VER=( `eval "${NEW_EXE}" 2>&1 | grep "VERSION"` )
		echo "${NEW_EXE_VER[*]}" && VERSION="${NEW_EXE_VER[5]}" && EDITION=`echo "${NEW_EXE_VER[0]}" | tr 'a-z' 'A-Z'`
	# Check dependencies
		echo "Checking Dependencies"
		CHECK_LIB
		LIBRARY=`ldd ${NEW_EXE}`
		echo "${LIBRARY}"
		OPTION_CODE "\n"
		echo "${LIBRARY}" | grep -i "not found" 
		[ $? -lt 1 ] && tput sgr0 && echo "Please install the necessary dependencies and try again" && exit 
		echo "looks okay" && tput sgr0;
		WAIT
	return;
	}

VALUE_CHANGE ()
	{
	# This is invoked from the CONF_DISPLAY Function, when option to change parameter is selected
		SHOW_BANNER
		echo "${MSG}"
		echo ""
		echo -e "Current Setting"
		OPTION_CODE ${VARS[$I]}; tput sgr0
		echo " = \"${!VARS[$I]}\""
		echo ""
		echo -ne "Please type the new value for "; OPTION_CODE "${VARS[$I]}"; tput sgr0; echo " and press Enter"
		read INPUT
		[ "x${INPUT}" != "x" ] && eval ${VARS[$I]}=`echo -e "${INPUT}"` && echo "value  changed!"
	return;
	}

# --------------------------------------------
# Character codes generated by the keypresses.
#
# uparrow=$'\x1b[A'
# arrowdown='\[B'
# arrowrt='\[C'
# arrowleft='\[D'
# insert='\[2'
# delete='\[3'
# --------------------------------------------	
	
	
	
REVIEW_CONF ()
	{
	# This function should always be called as REVIEW_CONF "FILE_NAME"
	# This is invoked from MAIN to display the current (latest) values from the FILE_NAME
		SHOW_BANNER
		[ "x${CONF}" == "x" ] && CONF=$1
		VAR=${VARS[$I]}
		# Locate the Contextual description for the VARiable
			MSG=`sed -n '/#-'"${VAR}"'/,/#--'"${VAR}"'/p' ${CONF}`
			MSG=`echo "${MSG}" | grep -i "#" | grep -iv "#-" | tr "#" ' '`
		OPTION_CODE "Reviewing Setup Parameters ( $I of $M )"; tput sgr0
		# Print the Contextual description for each VARiable
		echo "${MSG}"
		echo ""
		# Print the Current Value for the VARiable
		echo -e "Current Setting"
		OPTION_CODE ${VARS[$I]}; tput sgr0
		echo " = \"${!VARS[$I]}\""
		echo ""
		echo -e "Press \"C\" to Change value of this Parameter ( $I of $M )"
		echo -e "Press \"P\" to review Previous Parameter ( $I of $M )"
		echo -e "Press \"N\" to view Next Parameter ( $I of $M )"
		echo -e "Press \"S\" to confirm all Parameters are Set ( $I of $M )"
		read -e -n 1 INPUT
		A=`echo ${INPUT} | tr a-z A-Z`
		case ${A} in
		C)
			VALUE_CHANGE
			REVIEW_CONF
		;;
		P) 
			[ $I -gt 0 ] && let "I = $I - 1"
			REVIEW_CONF
		;;
		N)
			[ $I -lt $M ] && let "I = $I + 1"
			REVIEW_CONF
		;;
		S) return;
		;;
		*)
			REVIEW_CONF
		;;
		esac
	return;
	}

SHOW_SETTINGS ()
	{
		let "P = 0"
		while [ $P -le $M ] ;
		do
			eval "${VARS[${P}]}"="${!VARS[${P}]}"
			echo -ne "${VARS[${P}]}"
			echo -ne "="
			echo -e \""${!VARS[${P}]}"\"
			let "P += 1"
		done
	return;
	}

CHANGE_CONF ()
	{
	# This Function should always be called as CHANGE_CONF "FILE_NAME"
	# It will parse the "FILE_NAME" and appropriately change the parameters and save them into "FILE_NAME"
		let "P = 0"
		echo "Changing a total of $M Parameters from $1"
		while [ $P -le $M ] ;
		do
			sed '/^#/!s|'"${VARS[${P}]}"'=.*|'"${VARS[$[P]]}"'='"${!VARS[$P]}"'|' $1 1> $2 && cp -f $2 $1
			let "P += 1"
		done
	return;
	}

ADJUST_AND_OUT ()
	{
		SHOW_BANNER
		SHOW_SETTINGS
		echo ""
		echo -e "Press \"Y\" to copy the changed configuration into ${OLD_CONF}"
		echo -e "Press \"R\" to Restart all over from beginning \n"
		echo -e "Press \"Q\" to exit immediately, WITHOUT COMMITING ANY CHANGES"
		read -e -n 1 INPUT && A=`echo ${INPUT} | tr a-z A-Z`
		case ${A} in
			Y) cp ${NEW_CONF} ${OLD_CONF} ; exit
			;;
			R) MAIN
			;;
			Q) exit
			;;
			*) ADJUST_AND_OUT
			;;
		esac
	return;
	}
	
END_SUCCESS_1 ()
	{
		SHOW_BANNER
		OPTION_CODE  " Congratulations! You have Successfully installed SafeSquid ${NEW_EXE_VER[*]} "; tput sgr0
		echo ""
		echo " The various startup parameters of ${INSTANCE} are defined in: "; tput sgr0
		OPTION_CODE "${INSTANCE_DIR}/init.d/startup.conf" ; tput sgr0
		echo ""
		echo " To manage ${INSTANCE} do:"
		${RC_DIR}/${INSTANCE} usage
		OPTION_CODE "You can now conveniently manage the administrative and functional parameters from the SafeSquid WebGUI"; tput sgr0
		echo ""
		echo " To access your SafeSquid WebGUI .."
		echo -ne " Set your browser to use proxy at: "
		OPTION_CODE "${LISTEN}"; tput sgr0
		echo -ne "and open "
		OPTION_CODE "http://safesquid.cfg"; tput sgr0
		echo -e "\n\n""The various logs can be accessed at ${LOGDIR}/${INSTANCE}"
		echo -e "\n\n"
	return;
	}
	
	
END_SUCCESS ()
	{
		cat <<- _EOF
		
		`SHOW_BANNER`
		`OPTION_CODE` Congratulations! You have Successfully installed SafeSquid ${NEW_EXE_VER[*]} `tput sgr0`

		The various startup parameters of ${INSTANCE} are defined in: `tput sgr0`
		`OPTION_CODE` "${INSTANCE_DIR}/init.d/startup.conf `tput sgr0`

		To manage ${INSTANCE} do:
		`${RC_DIR}/${INSTANCE} usage`
		You can now conveniently manage the administrative and functional parameters from the SafeSquid WebGUI `tput sgr0`
		
		To access your SafeSquid WebGUI ..
		Set your browser to use proxy at: `OPTION_CODE` ${LISTEN} `tput sgr0`
		and open `OPTION_CODE`http://safesquid.cfg `tput sgr0`
		
		_EOF
		
	return;
	}
	
	
	
	
	
	

START_SERVICE ()
	{
		SHOW_BANNER
		cat <<- _EOF
			Press "S" if you would like to start your ${INSTANCE} now
			Press Any other key to simply exit
		_EOF
		

		read -e -n 1 INPUT && A=`echo ${INPUT} | tr a-z A-Z`
			case ${A} in
				S) ${RC_DIR}/${INSTANCE} restart
				;;
				*)
				;;
			esac
	return;
	}

GET_AGREEMENT ()
{
		SHOW_BANNER
		cat <<- _EOF
		
		Press "Y" if you find the End-User License Acceptable
		Press "A" To Read the End-User License Again
		
		Press "N" if you find the End-User License NOT Acceptable
		   and immediately abort the Installation Process
		_EOF
		

		read -e -n 1 INPUT && I_AGREE=`echo ${INPUT} | tr a-z A-Z`
		case ${I_AGREE} in
			Y) echo " Thank You for your patience and Accepting the End-User License ";
			;;
			A) READ_EULA
			GET_AGREEMENT
			;;
			N) echo " Terminating the ${PURPOSE} process " ; exit
			;;
			*) GET_AGREEMENT
			;;
		esac
	return;
}

READ_EULA ()
	{
		let "LINES = `eval wc -l < ./EULA.TXT`" 
		L=15
		H=15
		while [ $L -lt ${LINES} ] 
		do
		SHOW_BANNER
			echo -ne "\n\n"
			CONTENT=`eval head -n $L ./EULA.TXT | tail -n $H`
			echo "${CONTENT}"
			echo -e "\n\n""Displayed ${L} of  ${LINES} Lines"
			echo "Press \"B\" / \"F\" to move Back/Forward"
			echo "Press \"S\" When you have finished reading"
			read -e -n 1 M
				case ${M} in
					b|B) L=`expr $L - 1`
					;;
					f|F) L=`expr $L + 1`
					;;
					s|S) L=${LINES}
					;;
					*) 
					;;
				esac
		done
	return;
	}
		
COPY_BIN ()
	{
	# Take a backup of the existing bin directory and copy the new binary
		if [ -d ${INSTALL_DIR}/bin/ ] ; then 
			echo "Backing up Contents from your old INSTALL_DIR ${INSTALL_DIR} into ${BACKUP_DIR}/bin" 
			[ -d ${BACKUP_DIR}/bin/ ] || mkdir -p ${BACKUP_DIR}/bin/
			cp -rv ${INSTALL_DIR}/bin/* ${BACKUP_DIR}/bin/ && echo " ... Success!"
# This line was causing problems			
#			echo "Removing Contents from your old INSTALL_DIR ${INSTALL_DIR} " && rm -rf ${INSTALL_DIR}/*
#			echo "Removing Contents from your old INSTALL_DIR ${INSTALL_DIR} "
		else
			echo -ne "Creating ${INSTALL_DIR}/bin/ " && mkdir -p ${INSTALL_DIR}/bin/ && echo " ... Success!"
		fi
		echo -ne "Copying `basename ${NEW_EXE}` to ${INSTALL_DIR}/bin/ " && cp -vf ${NEW_EXE} ${INSTALL_DIR}/bin/ && echo " ... Success!" && ln -fs ${INSTALL_DIR}/bin/`basename ${NEW_EXE}` ${INSTALL_DIR}/bin/safesquid
		echo -ne "Copying ${CWD}/bin/section.xml to ${INSTALL_DIR}/bin/section.xml " 
		cp -f ${CWD}/bin/section.xml ${INSTALL_DIR}/bin/section.xml && echo " ... Success!" || echo " ... Failure!"
		
	# Take a backup of the existing modules directory and copy the new modules
		if [ -d ${CWD}/modules/ ] ; then
			if [ -d ${MODDIR}/ ] ; then
				echo "Backing up the contents of your module directory ${MODDIR} into ${BACKUP_DIR}/modules/"
				mkdir -p ${BACKUP_DIR}/modules && cp -rv ${MODDIR}/* ${BACKUP_DIR}/modules/
			fi
			mkdir ${MODDIR}/
			echo "Copying New modules into ${MODDIR}"
			cp -rv ${CWD}/modules/* ${MODDIR}/
			echo "Setting up soft-links for add-on modules"
			ln -fs ${MODDIR}/*/*.so ${MODDIR}/
			ln -fs ${MODDIR}/*/*.xml ${MODDIR}/
			
			echo "checking if any of the modules needs any additional fixing"
			
			for FX_SH in ${MODDIR}/*/modset.sh
			do 
				echo "executing ${FX_SH}"
				${FX_SH}
				cd ${CWD}
			done
				
			
			
		fi
	WAIT
	return;
	}

COPY_INST ()
	{
	# Merge the existing config xml into the installation source
		if [ -f ${CONFIG_FILE} ] ; then
			echo "Merging ${CONFIG_FILE} into Upgrade" 
			cp -fp ${CONFIG_FILE} ${CWD}/skel/`basename ${CONFIG_FILE}` && echo " ... Success"
		fi

	# If Instance Directory exists then Backup its contents
	# Else, create the new Instance directory
		if [ -d ${INSTANCE_DIR} ] ; then 
			echo "Backing up contents from your Old INSTANCE_DIR ${INSTANCE_DIR} into ${BACKUP_DIR}"
			cp -rv ${INSTANCE_DIR}/* ${BACKUP_DIR}/ && echo "contents from ${INSTANCE_DIR} backed-up to ${BACKUP_DIR}/"
		else
			echo "Building your new ${INSTANCE_DIR}" && mkdir -p ${INSTANCE_DIR} && echo " ... Success!"
		fi
	
	# Merge new Instance Directory contents into the INSTANCE_DIR
		#	echo -ne "Copying contents to ${INSTANCE_DIR} " && cp -rv ${CWD}/skel/* ${INSTANCE_DIR}/ && echo " ... Success!" 
			echo -e "Copying contents to ${INSTANCE_DIR} \n\n" && cp -i -rv ${CWD}/skel/* ${INSTANCE_DIR}/ 
			
			
	# Ensure restoration of old config xml file at the desired location
			echo "Re-checking presence of your config xml" && cp ${CWD}/skel/`basename ${CONFIG_FILE}` ${CONFIG_FILE} && echo " ... ok!" || ( echo " ... Failure!" ; exit )

	# Check the Instance has been properly created and set-up.
			echo -e "Checking our setup of ${INSTANCE} " && chmod 755 ${INSTANCE_DIR}/init.d/init.file && ${INSTANCE_DIR}/init.d/init.file dircheck
		
	# Provide mechanism to adjust startup.conf, when the user wants a readjustment in future.
			cp $0 ${INSTANCE_DIR}/adjust.sh && echo "Please use ${INSTANCE_DIR}/adjust.sh to adjust parameters of ${INSTANCE} in future" || ( echo " ... Failure!" ; exit )
	WAIT
	return;
	}
	
	
MAIN ()
	{

	# If this script has been called adjust.sh, then it is only supposed to adjust a startup.conf
	# So do not waste user's time querying about irrelevant stuff.
		
		if [ "x${IAM}" == "xadjust.sh" ] ; 
		then 
			PURPOSE="ADJUST"
			[ -f ${CWD}/init.d/startup.conf ] && OLD_CONF=${CWD}/init.d/startup.conf
		else
			[ "x${NEW_CONF}" == "x" ] && [ "x$1" != "x" ] && NEW_CONF=$1
			[ "x${NEW_CONF}" == "x" ] && [ -f ${CWD}/install.conf ] && NEW_CONF=${CWD}/install.conf
			[ "x${PURPOSE}" == "x" ] && QUERY_PURPOSE		
		fi

		
		
		
	# IF THE PURPOSE IS TO FRESH INSTALL A NEW BINARY OF SAFESQUID, THEN,
	# IT MIGHT BE BETTER TO CHECK THE MINIMAL COLLATERALS HERE,
	# RATHER THAN DO ALL THE SETTINGS AND DISCOVER, WE CAN'T ACTUALLY PROCEED
		if [ "x${PURPOSE}" == "xFRESH_INSTALL" ] || [ "x${PURPOSE}" == "xUPGRADE" ] ; then
			CHECK_NEW
			READ_EULA
			GET_AGREEMENT
	# Whenever we need to backup we will do it in BACKUP directory
			[ -d ${BACKUP_DIR} ] && rm -rf ${BACKUP_DIR}/* || mkdir -p ${BACKUP_DIR}/
		fi

		
		if [ "x${PURPOSE}" == "xUPGRADE" ] ; then
			QUERY_OLD_CONF
			INSTANCE_DIR=$(dirname $(dirname "${OLD_CONF}"))
	# GET THE PARAMETERS IN THE OLD CONF FILE
				READ_CONF "${OLD_CONF}"
	# IMPORT (MERGE) THE SETTINGS INTO NEW CONF FILE
				> ${CWD}/new.conf
				CHANGE_CONF ${NEW_CONF} ${CWD}/new.conf
				NEW_CONF="${CWD}/new.conf"
		fi
		
		if [ "x${PURPOSE}" == "xADJUST" ] ; then
			[ "x$1" != "x" ] && OLD_CONF="$1" || QUERY_OLD_CONF
			cp ${OLD_CONF} ${CWD}/new.conf
			NEW_CONF="${CWD}/new.conf"
		fi
		
		[ "x${PURPOSE}" == "xFRESH_INSTALL" ] && cp -f ${NEW_CONF} ${CWD}/new.conf && NEW_CONF="${CWD}/new.conf"
		
	# IN ANY CASE REVIEW & CHANGE PARAMETERS IN NEW_CONF !!
		READ_CONF "${NEW_CONF}"
		let "I = 0" ; REVIEW_CONF ${NEW_CONF}
		> ${CWD}/new2.conf
		echo "Incorporating your preferences"
		SHOW_SETTINGS > ${CWD}/new2.conf
	# Recalculate the values of the various parameters.
		PARAMS=`grep -ivE "#" ${CWD}/new2.conf`; eval "${PARAMS}"
		CHANGE_CONF ${NEW_CONF} ${CWD}/new2.conf
		NEW_CONF="${CWD}/new2.conf"
		WAIT
			
	# WE HAVE MADE ALL THE CONF SETTINGS,
	# SO IF THE PURPOSE WAS JUST TO ADJUST PARAMETERS, JUST SHOW THE SETTINGS, COMMIT CHANGES & GET OUT!
		[ "x${PURPOSE}" == "xADJUST" ] && ADJUST_AND_OUT

	# If the purpose is to upgrade or do a fresh installation, 
	# let us store the parameters in the "installation source" directory.
	# And that is what we will refer to as the startup.conf, henceforth.
		if [ "x${PURPOSE}" == "xUPGRADE" ] || [ "x${PURPOSE}" == "xFRESH_INSTALL" ] ; then
			sed -n '/#++STARTUP.CONF++#/,/#++++STARTUP.CONF++++#/p' ${NEW_CONF} > ${CWD}/skel/init.d/startup.conf
			NEW_CONF="${CWD}/skel/init.d/startup.conf"
		fi
	WAIT
	# Finally do the things, that are the objectives of the Fresh Installation or Upgrades.
		if [ "x${PURPOSE}" == "xUPGRADE" ] || [ "x${PURPOSE}" == "xFRESH_INSTALL" ]; then
			COPY_BIN
			COPY_INST
			END_SUCCESS
			START_SERVICE
		fi
	return;
	}
	
MAIN $*
exit